window.Gleam.initSearch([{"doc":"aragorn2","title":"aragorn2","content":"# aragorn2\n\n[![Package Version](https://img.shields.io/hexpm/v/aragorn2)](https://hex.pm/packages/aragorn2)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/aragorn2/)\n![Erlang-compatible](https://img.shields.io/badge/target-erlang-b83998)\n\nSecure password hashing, powered by Argon2.\n\n## Platform support\n\n`aragorn2` uses a [NIF](https://www.erlang.org/doc/system/nif) to provide the Argon2 implementation. This NIF leverages the [`argon2`](https://crates.io/crates/argon2) crate, which is a pure Rust implementation of Argon2.\n\nPrecompiled binaries are available for the following platforms for your convenience:\n\n| OS      | Architecture(s) |\n| ------- | --------------- |\n| Linux   | x86_64          |\n| macOS   | aarch64, x86_64 |\n| Windows | x86_64          |\n\n## Installation\n\n```sh\ngleam add aragorn2\n```\n\n## Usage\n\n```gleam\nimport aragorn2\nimport gleam/io\n\npub fn main() {\n  // Create a hasher to use to hash and verify passwords.\n  //\n  // You can customize the hasher&#39;s parameters, but be mindful of the security\n  // implications! The defaults are based on the current OWASP recommendations,\n  // so don&#39;t change them unless you know what you are doing.\n  let hasher = aragorn2.hasher()\n\n  // Provide a plaintext password to be hashed:\n  let assert Ok(hashed_password) =\n    aragorn2.hash_password(hasher, &lt;&lt;&quot;correct horse battery staple&quot;:utf8&gt;&gt;)\n  // This will return a hashed password that you can store (e.g., in your database).\n  // Reminder: Don&#39;t print out your hashed passwords in production.\n  io.debug(hashed_password)\n  // &quot;$argon2id$v=19$m=19456,t=2,p=1$SgDirmQl/Revk9+l7XtpZw$fz3xDI6cocCYpNB63FmMV4PhRpRTBK8KMuhYaWnAIKc&quot;\n\n  // When a user enters their candidate password, check it against the hashed\n  // password.\n  //\n  // When the candidate password does not match, an `Error` will be returned.\n  case\n    aragorn2.verify_password(\n      hasher,\n      candidate: &lt;&lt;&quot;wrong password&quot;:utf8&gt;&gt;,\n      hash: &lt;&lt;hashed_password:utf8&gt;&gt;,\n    )\n  {\n    Ok(Nil) -&gt; io.println(&quot;You&#39;re in!&quot;)\n    Error(Nil) -&gt; io.println(&quot;Oops, wrong password!&quot;) // &lt;--\n  }\n\n  // When the password does match, an `Ok` will be returned.\n  case\n    aragorn2.verify_password(\n      hasher,\n      candidate: &lt;&lt;&quot;correct horse battery staple&quot;:utf8&gt;&gt;,\n      hash: &lt;&lt;hashed_password:utf8&gt;&gt;,\n    )\n  {\n    Ok(Nil) -&gt; io.println(&quot;You&#39;re in!&quot;) // &lt;--\n    Error(Nil) -&gt; io.println(&quot;Oops, wrong password!&quot;)\n  }\n}\n```\n","url":"index.html"},{"doc":"aragorn2","title":"Algorithm","content":"pub type Algorithm {\n  Argon2d\n  Argon2i\n  Argon2id\n}\n An Argon2 hashing algorithm.\nArgon2d\n Argon2d is faster and uses data-depending memory access, which makes it highly resistant against GPU cracking attacks and suitable for applications with no threats from side-channel timing attacks (eg. cryptocurrencies).\n\nArgon2i\n Argon2i instead uses data-independent memory access, which is preferred for password hashing and password-based key derivation, but it is slower as it makes more passes over the memory to protect from tradeoff attacks.\n\nArgon2id\n Argon2id is a hybrid of Argon2i and Argon2d, using a combination of data-depending and data-independent memory accesses, which gives some of Argon2i&#39;s resistance to side-channel cache timing attacks and much of Argon2d&#39;s resistance to GPU cracking attacks.\n\n This is the default algorithm.\n\nSynonyms:\naragorn2.Algorithm\naragorn2 Algorithm","url":"aragorn2.html#Algorithm"},{"doc":"aragorn2","title":"Hasher","content":"pub opaque type Hasher\n An Argon2 password hasher.\n\nSynonyms:\naragorn2.Hasher\naragorn2 Hasher","url":"aragorn2.html#Hasher"},{"doc":"aragorn2","title":"hash_password","content":"pub fn hash_password(\n  hasher: Hasher,\n  password: BitArray,\n) -&gt; Result(String, Nil)\n Returns the hash of the given password.\nSynonyms:\naragorn2.hash_password\naragorn2 hash_password","url":"aragorn2.html#hash_password"},{"doc":"aragorn2","title":"hasher","content":"pub fn hasher() -&gt; Hasher\n Returns the default `Hasher`.\nSynonyms:\naragorn2.hasher\naragorn2 hasher","url":"aragorn2.html#hasher"},{"doc":"aragorn2","title":"verify_password","content":"pub fn verify_password(\n  hasher: Hasher,\n  candidate candidate_password: BitArray,\n  hash hashed_password: BitArray,\n) -&gt; Result(Nil, Nil)\n Returns whether the candidate password matches the given hashed password.\nSynonyms:\naragorn2.verify_password\naragorn2 verify_password","url":"aragorn2.html#verify_password"},{"doc":"aragorn2","title":"with_hash_length","content":"pub fn with_hash_length(\n  hasher: Hasher,\n  hash_length: Int,\n) -&gt; Hasher\n Sets the output hash length of the `Hasher`.\n\n This is the length of the output hash, in bytes.\n\n Must be a value between `4` and `(2^32) - 1`.\n\n The default value is `32`.\nSynonyms:\naragorn2.with_hash_length\naragorn2 with_hash_length","url":"aragorn2.html#with_hash_length"},{"doc":"aragorn2","title":"with_memory_cost","content":"pub fn with_memory_cost(\n  hasher: Hasher,\n  memory_cost: Int,\n) -&gt; Hasher\n Sets the memory cost of the `Hasher`.\n\n This is the memory size, expressed in kibibytes (KiB).\n\n Must be a value between `8 * parallelism` and `(2^32) - 1`.\n\n The default value is `19,456` (19KiB).\nSynonyms:\naragorn2.with_memory_cost\naragorn2 with_memory_cost","url":"aragorn2.html#with_memory_cost"},{"doc":"aragorn2","title":"with_parallelism","content":"pub fn with_parallelism(\n  hasher: Hasher,\n  parallelism: Int,\n) -&gt; Hasher\n Sets the parallelism of the `Hasher`.\n\n This is the number of threads that will be used when hashing a password.\n\n Must be a value between `1` and `(2^32) - 1`.\n\n The default value is `1`.\nSynonyms:\naragorn2.with_parallelism\naragorn2 with_parallelism","url":"aragorn2.html#with_parallelism"},{"doc":"aragorn2","title":"with_time_cost","content":"pub fn with_time_cost(hasher: Hasher, time_cost: Int) -&gt; Hasher\n Sets the time cost of the `Hasher`.\n\n This is the number of iterations that will be performed when hashing a password.\n\n Must be a value between `1` and `(2^32) - 1`.\n\n The default value is `2`.\nSynonyms:\naragorn2.with_time_cost\naragorn2 with_time_cost","url":"aragorn2.html#with_time_cost"},{"doc":"aragorn2","title":"aragorn2","content":" Aragorn2 provides secure password hashing, powered by Argon2.\n\n From the [Argon2 repo](https://github.com/P-H-C/phc-winner-argon2):\n\n &gt; Argon2 is a password-hashing function that summarizes the state of the art in the design of memory-hard functions and can be used to hash passwords for credential storage, key derivation, or other applications.\n &gt;\n &gt; It has a simple design aimed at the highest memory filling rate and effective use of multiple computing units, while still providing defense against tradeoff attacks (by exploiting the cache and memory organization of the recent processors).\n &gt;\n &gt; Argon2 has three variants: Argon2i, Argon2d, and Argon2id. Argon2d is faster and uses data-depending memory access, which makes it highly resistant against GPU cracking attacks and suitable for applications with no threats from side-channel timing attacks (eg. cryptocurrencies). Argon2i instead uses data-independent memory access, which is preferred for password hashing and password-based key derivation, but it is slower as it makes more passes over the memory to protect from tradeoff attacks. Argon2id is a hybrid of Argon2i and Argon2d, using a combination of data-depending and data-independent memory accesses, which gives some of Argon2i&#39;s resistance to side-channel cache timing attacks and much of Argon2d&#39;s resistance to GPU cracking attacks.\n &gt;\n &gt; Argon2i, Argon2d, and Argon2id are parametrized by:\n &gt;\n &gt; - A **time cost**, which defines the amount of computation realized and therefore the execution time, given in number of iterations\n &gt; - A **memory cost**, which defines the memory usage, given in kibibytes\n &gt; - A **parallelism degree**, which defines the number of parallel threads","url":"aragorn2.html"}]);